// HPRINT.C    Written by Scott R. Houck
//
// HP Laserjet utility to print a file with line numbers and a heading,
// in condensed mode, 8 lpi, and alternating white and shaded rows (like
// old computer paper).
//
// Version 1.2 (2/21/91)
//   In the heading, print the file modification date and time, not the
//   current system time.
//
// Version 1.1 (2/20/91)
//   Cleaned the code up a bit and added the /2 option
//
// Version 1.0 (2/19/91)
//   Initial release

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <io.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#define STARTPOS  75       // Two lines for the heading
#define MAXLINE   250      // Maximum line from the file
#define MAXCHARS  134      // Maximum chars that print in condensed mode

char *dow[] = { "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday",
   "Friday", "Saturday" };

char *mth[] = { "January", "February", "March", "April", "May", "June",
   "July", "August", "September", "October", "November", "December" };

struct stat stbuf;
struct tm *ft;    // file time
char ampm;
int  hours;
FILE *infile, *outfile;
char drive [ _MAX_DRIVE ];
char dir   [ _MAX_DIR   ];
char fname [ _MAX_FNAME ];
char ext   [ _MAX_EXT   ];
char filename[13];
char filetext[30];
int  timetextlen;
char timetext[80];
int  filefieldlen;
int  i;
char line [ MAXLINE ];
long lineno = 0;
char format[20];
char pagetext[20];
int  pageno = 0;

// Default print controls for 3 print lines per shaded box.
// For a 10 inch print area at 300 dpi, there are 3000 dots of
// vertical printing area.  At 8 lpi, 80 lines print.  Therefore,
// there are 3000 / 80 = 37.5 dots per line.

int rowheight   = 112;  // 3 * 37.5, truncated
int twiceheight = 225;  // 2 * 3 * 37.5
int numboxes    = 13;   // (3000 - STARTPOS) / twiceheight

main(int argc, char *argv[])
{
   fprintf(stderr, "HPRINT v1.2 by Scott R. Houck\n");

   if (argc < 2)
      {
      fprintf(stderr, "Usage:  HPRINT filename [/2]\n");
      fprintf(stderr, "Use /2 to print 2 lines per shaded area\n");
      fprintf(stderr, "rather than the default of 3 lines.\n");
      exit(0);
      }

   // Get the file's date and time of last modification
   if (stat(argv[1], &stbuf) == -1)
      {
      if (errno == ENOENT)
         fprintf(stderr, "File '%s' not found\n", argv[1]);
      else
         fprintf(stderr,
            "Error obtaining file status information for '%s'\n", argv[1]);
      exit(1);
      }

   // Convert the field st_atime, which is of type time_t, into a type
   // of struct tm.
   ft = localtime(&stbuf.st_atime);

   // Convert the 24-hour format into 12-hour format
   hours = ft->tm_hour;
   ampm  = hours < 12 ? 'a' : 'p';
   hours = (hours + 11) % 12 + 1;

   timetextlen = sprintf(timetext,
      "Last modified: %s, %s %d, %d at %d:%02d %cm",
      dow[ft->tm_wday], mth[ft->tm_mon], ft->tm_mday,
      ft->tm_year + 1900, hours, ft->tm_min, ampm);

   filefieldlen = (MAXCHARS - timetextlen) / 2;

   // Set up the format for the heading
   sprintf(format, "%%-%ds%%s%%%ds\n\n", filefieldlen,
      MAXCHARS - filefieldlen - timetextlen - 1);

   if ((infile = fopen(argv[1], "r")) == NULL)
      {
      fprintf(stderr, "Cannot open '%s'\n", argv[1]);
      exit(1);
      }

   if (isatty(fileno(stdout)))
      outfile = stdprn;
   else
      outfile = stdout;

   // Make sure LF is translated into CR-LF on output
   setmode(fileno(outfile), O_TEXT);

   _splitpath(argv[1], drive, dir, fname, ext);
   sprintf(filename, "%s%s", fname, ext);
   strupr(filename);
   sprintf(filetext, "  File: %s", filename);

   fprintf(stderr, "Printing %s...", filename);

   if (argc > 2 && strcmp(argv[2], "/2") == 0)
      {
      rowheight = 75;      // 2 * 37.5
      twiceheight = 150;
      numboxes = 20;
      }

   // Reset
   fprintf(outfile, "\033E");

   // Set condensed print
   fprintf(outfile, "\033(s8.5v16.66H");

   // Set 8 lines per inch
   fprintf(outfile, "\033&l6C");  // 6 -> 6/48 = 1/8

   // Define macro ID #1
   fprintf(outfile, "\033&f1Y");

   // Start macro definition
   fprintf(outfile, "\033&f0X");

   fprintf(outfile, "\033*c2400A");           // Width of rectangle
   fprintf(outfile, "\033*c%dB", rowheight);  // Height of rectangle
   fprintf(outfile, "\033*c10G");             // 10% fill

   // Print the boxes
   for (i = 0; i < numboxes; i++)
      {
      fprintf(outfile, "\033*p0x%dY", i * twiceheight + STARTPOS);
      fprintf(outfile, "\033*c2P");           // Print with shaded fill
      }

   // Stop macro definition
   fprintf(outfile, "\033&f1X");

   // Enable macro #1 for automatic overlay
   fprintf(outfile, "\033&f1y4X");

   while (fgets(line, MAXLINE, infile))
      {
      if (lineno % 78 == 0)
         {
         // Print the heading
         sprintf(pagetext, "Page %d", ++pageno);
         fprintf(outfile, format, filetext, timetext, pagetext);
         }
      fprintf(outfile, "%6ld: %s", ++lineno, line);
      }

   fclose(infile);

   // Send a final form feed unless a full page had already been printed
   if (lineno % 78 != 0)
      fprintf(outfile, "\f");

   // Delete the macro
   fprintf(outfile, "\033&f1y8X");

   // Reset
   fprintf(outfile, "\033E");

   fprintf(stderr, " done!\n");
}
